# get the correct string type for python 2/3
try:
    isinstance("", basestring)
    string_type = basestring

except NameError:
    string_type = str

switch_on = True
switch_off = False

def none_or(type):
    """Pseudo type casting any none-value to the given type
    """
    if isinstance(type, list):
        raise ValueError("cannot make a list-type none-able.")
    
    if isinstance(type, tuple):
        return NoneAbleTuple(type)
    
    def func(val):
        if val is None:
            return val
        
        return type(val)
    
    return func

def custom_bool(val):
    """Pseudo type to handle yes/no strings as booleans
    """
    if isinstance(val, string_type):
        val = val.lower()
        assert val in ["yes", "no", "true", "false"]
        return (val == "yes") or (val == "true")
    
    return bool(val)

class TupleType(object):
    """Pseudo types used to handle typed-tuples
    """
    def __init__(self, spec):
        self.spec = tuple(spec)
    
    def __len__(self):
        return len(self.spec)
    
    def __iter__(self):
        return iter(self.spec)
    
    def __call__(self, val):
        assert len(val) == len(self)    
        return tuple(t(v) for (t,v) in zip(self,val))

class NoneAbleTuple(TupleType):
    """Pseudo type to handle typed-tuples and none
    """
    def __call__(self, val):
        if val is None:
            return val
        
        return super(NoneAbleTuple, self).__call__(val)

def translate_types(typespec):
    """Replace the annotation types by implementation specific variants.
    """
    type_replace = {bool: custom_bool}

    def maptype(type):
        if isinstance(type, list):
            assert len(type) == 1, "list types have to be of the form [type]"
            return [maptype(type[0])]

        elif isinstance(type, set):
            assert len(type) != 0, "set types have to have a single item"
            return type

        elif isinstance(type, tuple):
            return TupleType(maptype(t) for t in type)

        return type_replace.get(type, type)

    return {arg: maptype(type) for (arg, type) in typespec.items()}
