from .util import defaultopt

def types(**typespec):
    """Add type annotations to function arguments.
    """
    def decorator(func):
        defaultopt(func, "types", {}).update(typespec)
        return func
        
    return decorator

def help(**helpspec):
    """Add help texts to function arguments. 
    """
    def decorator(func):
        defaultopt(func, "help", {}).update(helpspec)
        return func
    
    return decorator

def arg(name, type, helptext=None):
    """Shortcut to set the type and help of an argument.
    """
    def decorator(func):
        types(**{name: type})(func)
        
        if helptext is not None:
            help(**{name: helptext})(func)
        
        return func
    
    return decorator

def kwargs(**kwargs):
    """Add additional keyword arguments to the function signature.
    
    The type and help attributes of the added arguments can be set as
    usual with the type, help or arg decorator. 
    
    Usage::
    
        @argmap.kwargs(a="a", b="b")
        def func(**options):
            print options["a"]
            print options["b"]
    
    """
    def decorator(func):
        defaultopt(func, "kwargs", {}).update(kwargs)
        return func
    
    return decorator
