import argmap
import sys

from unittest import TestCase, main
from argmap.util import get_common_type

class TestChoices(TestCase):
    def test_common_type(self):
        self.assertIs(int, get_common_type((1, 2, 3)))
        self.assertIs(float, get_common_type((1.0, 2.0, 3.0)))
        self.assertIs(str, get_common_type(("1", "2", "3")))
    
    def test_failure_for_different_types(self):
        """types need to matche exactly"""
        self.assertRaises(TypeError, get_common_type, (1, 1.0))
    
    def test_failure_for_empty_sequence(self):
        """types need to matche exactly"""
        self.assertRaises(ValueError, get_common_type, [])
    
    def test_choice_pos_arg(self):
        @argmap.arg("choice", {1, 2, 3})
        def func(choice):
            return choice
        
        self.assertEqual(2, argmap.argmap(func, ["2"]))
    
    def test_choice_failure(self):
        @argmap.arg("choice", {1, 2, 3})
        def func(choice):
            return choice
        
        self.assertRaises(SystemExit, argmap.argmap, func, ["4"])
    
    def test_choice_kwarg(self):
        @argmap.arg("choice", {1, 2, 3})
        def func(choice=4):
            return choice
        
        self.assertEqual(2, argmap.argmap(func, ["--choice", "2"]))

if sys.version_info[0] >= 3:
    from .impl3.cases import TestChoices3

if __name__ == "__main__":
    main()

