from unittest import TestCase, main
import argmap
import sys

class TestCompound(TestCase):
    def test_tuples(self):
        @argmap.types(pos=(float, float))
        def func(pos):
            return pos
        
        self.assertTupleEqual(argmap.argmap(func, ["2.0", "3.0"]), (2.0, 3.0))
    
    def test_list(self):
        @argmap.types(a=[int])
        def func(a=[]):
            return a
        
        self.assertSequenceEqual(argmap.argmap(func, ["--a", "1", "--a", "2"]),
                                 [1,2])
        
        self.assertSequenceEqual(argmap.argmap(func, []), [])
    
    def test_list_default(self):
        @argmap.types(a=[int])
        def func(a=[2,3]):
            return list(a)
        
        self.assertSequenceEqual(argmap.argmap(func, []), [2,3])
        self.assertSequenceEqual(argmap.argmap(func, ["--a", "1"]), [1])
    
    def test_list_of_tuples(self):
        @argmap.arg("it", [(str, int)])
        def func(it=[]):
            return it
        
        self.assertListEqual(argmap.argmap(func, ["--it", "a", "1", 
                                                  "--it", "b", "2"]),
                             [("a", 1), ("b", 2)])
    
    def test_none_or_tuple(self):
        @argmap.arg("a", argmap.none_or((str, int)))
        def func(a=None):
            return a
        
        self.assertIsNone(argmap.argmap(func, []))
        self.assertTupleEqual(argmap.argmap(func, ["--a", "b", "20"]), ("b", 20))

    def test_tuple_of_none_or_int_correct(self):
        @argmap.arg("a", (argmap.none_or(int), argmap.none_or(int)))
        def func(a=(None,None)):
            return a

        self.assertEqual(argmap.argmap(func, []), (None, None))
        self.assertEqual(argmap.argmap(func, ["--a", "20", "30"]), (20, 30))

    def test_tuple_of_none_or_int_failing(self):
        """Incorrectly annotated, default arguments can break argument parsing.

        See test_tuple_of_none_or_int_correct for the correct annotation.
        """
        @argmap.arg("a", (int, int))
        def func(a=(None,None)):
            return a

        with self.assertRaises(TypeError):
            argmap.argmap(func, [])

if sys.version_info[0] >= 3:
    from .impl3.cases import TestCompound3

if __name__ == "__main__":
    main()
