from unittest import TestCase, main
import argmap
import sys

class TestSimple(TestCase):
    def test_posargs(self):
        def func(a, b):
            return dict(a=a, b=b)
        
        self.assertDictEqual(argmap.argmap(func, ["hello", "world"]), 
                             dict(a="hello", b="world"))    
    
    def test_posargs_order(self):
        def func(a, b):
            return [a, b]
        
        self.assertListEqual(argmap.argmap(func, ["hello", "world"]), 
                             ["hello", "world"])
    
    def test_posargs_order_annotate(self):
        @argmap.arg("a", str, "a")
        @argmap.arg("b", str, "b")
        def func(a, b):
            return [a, b]
    
        self.assertListEqual(argmap.argmap(func, ["hello", "world"]), 
                             ["hello", "world"])
    
    def test_types(self):
        @argmap.types(a=int, b=float)
        def func(a, *b):
            return dict(a=a, b=list(b))
        
        res = argmap.argmap(func, ["1", "2.5", "3.5"])
        
        self.assertSetEqual(set(res.keys()), {"a", "b"})
        self.assertEqual(res["a"], 1)
        self.assertSequenceEqual(res["b"], [2.5, 3.5])
    
    def test_bool(self):
        @argmap.types(a=bool)
        def func(a):
            return a
    
        self.assertEqual(argmap.argmap(func, ["yes"]), True)
        self.assertEqual(argmap.argmap(func, ["no"]), False)
    
    def test_switch_onoff(self):
        @argmap.types(a=argmap.switch_on, b=argmap.switch_off)
        def func(a, b):
            return a, b
        
        self.assertTupleEqual(argmap.argmap(func, []), (False, True))
        self.assertTupleEqual(argmap.argmap(func, ["--a"]), (True, True))
        self.assertTupleEqual(argmap.argmap(func, ["--b"]), (False, False))
        self.assertTupleEqual(argmap.argmap(func, ["--b", "--a"]), (True, False))

if sys.version_info[0] >= 3:
    from .impl3.cases import TestSimple3

if __name__ == "__main__":
    main()
