from unittest import TestCase, main
import argmap
import sys

class TestVarArgs(TestCase):
    def test_varargs(self):
        def func(*args):
            return args
        
        self.assertSequenceEqual(argmap.argmap(func, 
                                               ["hello", "strange", "world"]), 
                             ["hello", "strange", "world"])
    
    def test_varargs_posargs_optargs(self):
        def func(a, b="", *c):
            return (a,b,c)
        
        a,b,c = argmap.argmap(func, ["--b", "abc", "def", "g", "h", "i"])
        self.assertEqual(a, "def")
        self.assertEqual(b, "abc")
        self.assertSequenceEqual(c, ["g", "h", "i"])
    
    def test_varargs_simple_type(self):
        @argmap.arg("args", float)
        def func(*args):
            return args
        
        self.assertSequenceEqual(argmap.argmap(func, ["1", "2", "3"]), 
                                 [1, 2, 3])
    
    def test_varargs_tuple_type(self):
        @argmap.arg("args", (str, float))
        def func(*args):
            return args
        
        self.assertSequenceEqual(argmap.argmap(func, ["1", "2", "3", "4"]), 
                                 [("1", 2), ("3", 4)])
    
    def test_varargs_underscored_name(self):
        def func(*func_args):
            return func_args
        
        self.assertSequenceEqual(argmap.argmap(func, 
                                               ["hello", "strange", "world"]), 
                             ["hello", "strange", "world"])
    
if sys.version_info[0] >= 3:
    from .impl3.cases import TestVarArgs3

if __name__ == "__main__":
    main()
