def argnamefmt(arg):
    """Returns an argument name  for the commandline for a function argument.
    
    Replaces underscores by a hyphen.
    """
    return arg.replace("_", "-")

def getopt(func, name, default={}):
    """dict.get implementation for objects (with prefix _argmap_).
    """
    if name == "types":
        return getopt_types(func, default)

    try:
        return getattr(func, "_argmap_" + name)
    
    except AttributeError:
        return default

def getopt_types(func, default={}):
    """Support python 3 function annotations for the types argument
    """
    if not (hasattr(func, "__annotations__") or hasattr(func, "_argmap_types")):
        return default

    result = {}

    try:
        result.update(func.__annotations__)

    except AttributeError:
        pass

    try:
        result.update(func._argmap_types)

    except AttributeError:
        pass

    return result

def defaultopt(func, name, default):
    """dict.setdefault implementation for objects (with prefix _argmap_).
    """
    
    argname = "_argmap_" + name
    
    if not hasattr(func, argname):
        setattr(func, argname, default)
    
    return getattr(func, argname)

def get_common_type(values):
    if len(values) == 0:
        raise ValueError("Need at least a single value to get common type")
    
    first_type = values[0].__class__

    if not all(val.__class__ is first_type for val in values):
        raise TypeError("Could not determine a common type")
    
    return first_type

def get_description_and_epilog(func):
    if func.__doc__ is None:
        return None, None
    
    lines = func.__doc__.splitlines()
    lines = [line.strip() for line in lines]
    
    non_empty_count = sum(1 for line in lines
                          if line != "")
    
    if non_empty_count == 0:
        return None, None
    
    elif non_empty_count == 1:
        return lines[0], None
    
    epilog = "\n".join(lines[1:])
    epilog = epilog.strip()
    
    return lines[0], epilog
    
